# import the necessary packages
from .camgear import CamGear
import time

class VideoGear:
	"""
	This a common class to access any of the vidgear primary classes(i.e. PiGear and CamGear), separated by enablePiCamera flag. 
	This Class accepts different attributes and parameters based on the class that is being accessed currently.
	"""

	def __init__(self, source=0, enablePiCamera = False, resolution=(640, 480), framerate=25, logging = False, time_delay = 0, **options):
		
		if enablePiCamera:
			# only import the pigear module only if required
			from .pigear import PiGear

			# initialize the picamera stream by enabling PiGear Class
			self.stream = PiGear(resolution=resolution, framerate=framerate, logging = logging, time_delay = time_delay, **options)

		else:
			# otherwise, we are using OpenCV so initialize the webcam
			# stream by activating CamGear Class
			self.stream = CamGear(source=source, logging = logging, time_delay = time_delay, **options)

	def start(self):
		# start the threaded video stream
		return self.stream.start()

	def update(self):
		# grab the next frame from the stream
		self.stream.update()

	def read(self):
		# return the current frame
		return self.stream.read()

	def stop(self):
		# stop the thread and release any resources
		self.stream.stop()


# import the packages
from threading import Thread
from pkg_resources import parse_version
import logging

try:
	# import OpenCV Binaries
	import cv2

	# check whether OpenCV Binaries are 3.x+
	if parse_version(cv2.__version__) >= parse_version('3'):
		pass
	else:
		raise ImportError('OpenCV library version >= 3.0 is only supported by this library')

except ImportError as error:
	raise ImportError('Failed to detect OpenCV executables, install it with "pip install opencv-python" command.')

class PiGear:
	"""
	This class exclusively targets the Raspberry Pi Camera Modules such as OmniVision OV5647 Camera Module and Sony IMX219 Camera Module, 
	to obtain high FPS video by utilizing OpenCV and Picamera libraries with a bit of multithreading. But make sure to enable Raspberry Pi 
	hardware specific settings prior using this class.
	"""

	def __init__(self, resolution=(640, 480), framerate=25, logging = False, time_delay = 0, **options):

		try:
			import picamera
			from picamera.array import PiRGBArray
			from picamera import PiCamera
			#print(cv2.__version__)
		except ImportError as error:
			# Output expected ImportErrors.
			raise ImportError('Failed to detect Picamera executables, install it with "pip install picamera" command.')

		# initialize the picamera stream
		self.camera = PiCamera()
		self.camera.resolution = resolution
		self.camera.framerate = framerate

		try: 
			# apply attributes to source if specified
			for key, value in options.items():
				setattr(self.camera, key.strip(), value)
		except Exception as e:
			# Catch if any error occurred
			if logging:
				print(e)

		# enable rgb capture array thread and capture stream
		self.rawCapture = PiRGBArray(self.camera, size=resolution)
		self.stream = self.camera.capture_continuous(self.rawCapture,format="bgr", use_video_port=True)

		# applying time delay to warmup picamera only if specified
		if time_delay:
			import time
			time.sleep(time_delay)

		#thread initialization
		self.thread = None
		
		#frame initialization
		self.frame = None

		# enable logging if specified
		self.logging = logging

		# intialize termination flag
		self.terminate = False


	def start(self):

		# start the thread to read frames from the video stream
		self.thread = Thread(target=self.update, args=())
		self.thread.daemon = True
		self.thread.start()
		return self

	def update(self):
		# keep looping infinitely until the thread is terminated
		try:
			for stream in self.stream:
			# grab the frame from the stream and clear the stream in
			# preparation for the next frame
				if stream is None:
					if self.logging:
						print('The Camera Module is not working Properly!')
					self.terminate =True
				if self.terminate:
					break
				self.frame = stream.array
				self.rawCapture.seek(0)
				self.rawCapture.truncate()
		except Exception as e:
			if self.logging:
				logging.error(traceback.format_exc())
			self.terminate =True
			pass

		# release resource camera resources
		self.stream.close()
		self.rawCapture.close()
		self.camera.close()

	def read(self):
		# return the frame most recently read
		return self.frame

	def stop(self):
		# indicate that the thread should be terminated
		self.terminate = True
		# wait until stream resources are released (producer thread might be still grabbing frame)
		if self.thread is not None: 
			self.thread.join()
			#properly handle thread exit




# import the necessary packages
from threading import Thread
from pkg_resources import parse_version
import re

#Note: Remember, Not all parameters are supported by all cameras which is 
#one of the most troublesome part of the OpenCV library. Each camera type, 
#from android cameras to USB cameras to professional
#ones offer a different interface to modify its parameters. 
#Therefore there are many branches in OpenCV code to support 
#as many of them, but of course not all possible devices 
#are covered and thereby works.

#To check parameter values supported by your webcam, you can hook your camera
#to your Linux machine and use command `v4l2-ctl -d 0 --list-formats-ext` (where 0 is index of given camera)
#to list the supported video parameters and their values.

try:
	# import OpenCV Binaries
	import cv2
	# check whether OpenCV Binaries are 3.x+
	if parse_version(cv2.__version__) >= parse_version('3'):
		pass
	else:
		raise ImportError('OpenCV library version >= 3.0 is only supported by this library')

except ImportError as error:
	raise ImportError('Failed to detect OpenCV executables, install it with "pip install opencv-python" command.')

class CamGear:
	"""This class targets any common IP or USB Cameras(including Raspberry Pi Compatible), 
	Various Video Files Formats and Network Video Streams(Including Gstreamer Raw Video Capture Pipeline) 
	for obtaining high-speed real-time frames by utilizing OpenCV and multi-threading."""

	def __init__(self, source = 0, logging = False, time_delay = 0, **options):
		# initialize the camera stream and read the first frame

		self.stream = cv2.VideoCapture(source)

		try: 
			# try to apply attributes to source if specified
			for key, value in options.items():
				self.stream.set(self.capPropId(key.strip()),value)
		except Exception as e:
			# Catch if any error occurred
			if logging:
				print(e)

		(self.grabbed, self.frame) = self.stream.read()

		# applying time delay to warm-up webcam only if specified
		if time_delay:
			import time
			time.sleep(time_delay)

		# thread initialization
		self.thread=None

		# initialize termination flag
		self.terminate = False

	def start(self):
		# start the thread to read frames from the video stream
		self.thread = Thread(target=self.update, args=())
		self.thread.daemon = True
		self.thread.start()
		return self

	def capPropId(self, property):
		#Retrieves the Property's Integer(Actual) value. 
		return getattr(cv2, property)

	def update(self):
		# keep looping infinitely until the thread is terminated
		while True:
			# if the thread indicator variable is set, stop the thread
			if self.terminate:
				break

			# otherwise, read the next frame from the stream
			(self.grabbed, self.frame) = self.stream.read()

			#check for valid frames
			if not self.grabbed:
				#no frames received, then safely exit
				self.terminate = True
				
		#release resources
		self.stream.release()

	def read(self):
		# return the frame most recently read
		return self.frame

	def stop(self):
		# indicate that the thread should be terminate
		self.terminate = True

		# wait until stream resources are released (producer thread might be still grabbing frame)
		if self.thread is not None: 
			self.thread.join()
			#properly handle thread exit



import setuptools
from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='vidgear',
    packages=['vidgear','vidgear.gears'],
    version='0.1.3',
    description='A Lightweight Multi-Threaded OpenCV Video Processing Python Wrapper Library',
    license='MIT License',
    author='abhiTronix',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author_email='abhi.una12@gmail.com',
    url='https://github.com/abhiTronix/vidgear',
    download_url='https://github.com/abhiTronix/vidgear/tarball/0.1.3',
    keywords=['computer vision', 'multi-thread', 'opencv', 'opencv2', 'opencv4', 'picamera'],
    classifiers=[
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7'],
    python_requires='>=2.7',
    scripts=[],
    project_urls={  # Optional
        'Bug Reports': 'https://github.com/abhiTronix/vidgear/issues',
        'Funding': 'https://paypal.me/AbhiTronix?locale.x=en_GB',
        'Source': 'https://github.com/abhiTronix/vidgear',},
)
